from __future__ import absolute_import
from __future__ import division
import simplejson
from clint import textui

from loveseat.suite import Suite


class TestRunner:
    def __init__(self, suitefile):
        with open(suitefile, 'r') as f:
            content = simplejson.loads(f.read())

        self.suite = Suite(
            content['suite'],
            content['slug'],
            content['tests'],
        )

    def run(self):
        results = self.suite.run()

        textui.puts(textui.colored.blue(self.suite.name))

        for ag_result in results:
            column_results = ag_result.results
            column_width = 30

            output = [[textui.colored.magenta(ag_result.name), column_width]]
            output += map(lambda db: [textui.colored.magenta(db.split('/')[-1]), column_width],
                          column_results.keys())

            textui.puts(textui.columns(*output))

            metrics = ['count', 'avg', 'max', 'min']

            for m in metrics:
                output = [[m, column_width]]
                for r in column_results.items():
                    if m != 'count':
                        formatted = "{} ms".format(r[1][m] / 1000)
                    else:
                        formatted = unicode(r[1][m])
                    output.append([formatted, column_width])

                textui.puts(textui.columns(*output))

            textui.puts('\n')



from __future__ import absolute_import
import requests
from loveseat.result import Result


def read(database, id, **kwargs):
    params = kwargs.get('params', {})
    resp = requests.get(database,
                        params=params,
                        headers=kwargs.get('headers', {}))
    return Result(
        database=(kwargs.get('slug') or database),
        params=params,
        elapsed=resp.elapsed,
        test='read'
    )


def all_docs(database, **kwargs):
    params = kwargs.get('params', {})
    resp = requests.get("{database}/_all_docs".format(database=database),
                        params=params,
                        headers=kwargs.get('headers', {}))
    return Result(
        database=(kwargs.get('slug') or database),
        params=params,
        elapsed=resp.elapsed,
        test='all_docs'
    )


def view(database, view, **kwargs):
    params = kwargs.get('params', {})
    resp = requests.get("{database}/{view}".format(database=database, view=view),
                        params=params,
                        headers=kwargs.get('headers', {}))
    return Result(
        database=(kwargs.get('slug') or database),
        params=params,
        elapsed=resp.elapsed,
        test='view'
    )



from __future__ import absolute_import
import unittest
from os.path import abspath, dirname, join
from loveseat.couch_specs import CouchReadSpec


class TestCouchSpecs(unittest.TestCase):
    def test_multiple_databases(self):
        db_url = 'http://localhost:5984/commcarehq'
        db_dict = {
            'url': 'my://broken/url',
            'params': {
                'my': 'param'
            }
        }
        spec = {
            'name': 'dummy',
            'test': 'read',
            'databases': ['http://localhost:5984/commcarehq', db_dict],
            'params': {},
            'repeat': 100,
            'headers': {
                'silly': 'header'
            },
            'ids': ['000382ba-386f-4757-a80b-8a8aa62eec97']
        }

        read_spec = CouchReadSpec(spec)

        self.assertEqual(len(read_spec.databases), 2)
        self.assertEqual(read_spec.databases[0]['url'], db_url)
        self.assertEqual(read_spec.databases[0]['headers']['silly'], 'header')

        self.assertEqual(read_spec.databases[1]['url'], db_dict['url'])
        self.assertEqual(read_spec.databases[1]['headers']['silly'], 'header')
        self.assertEqual(read_spec.databases[1]['params']['my'], 'param')



from __future__ import absolute_import
import unittest
from os.path import abspath, dirname, join
from loveseat.test_runner import TestRunner

THISDIR = dirname(abspath(__file__))
BASE_PATH = join(THISDIR, 'data')


class TestTestRunner(unittest.TestCase):
    def setUp(self):
        self.suitefile = join(BASE_PATH, 'test_suite.json')

    def test_init_test_runner(self):
        runner = TestRunner(self.suitefile)

        self.assertEqual(runner.suite.slug, 'example_suite')
        self.assertEqual(runner.suite.name, 'Example Suite')
        self.assertEqual(len(runner.suite.tests), 4)



from __future__ import absolute_import
import unittest
from datetime import timedelta
from loveseat.aggregated_result import AggregatedResult
from loveseat.result import Result


class TestAggregatedResult(unittest.TestCase):

    def setUp(self):
        self.resultOne = Result(database='a', elapsed=timedelta(0, 0, 2))
        self.resultTwo = Result(database='a', elapsed=timedelta(0, 0, 4))
        self.resultThree = Result(database='b', elapsed=timedelta(0, 0, 5))

    def test_aggregated_result(self):
        ag = AggregatedResult('example')

        ag.add_results([self.resultOne, self.resultTwo, self.resultThree])
        self.assertEqual(ag.results['a']['avg'], 3)
        self.assertEqual(ag.results['a']['max'], 4)
        self.assertEqual(ag.results['a']['min'], 2)
        self.assertEqual(ag.results['a']['count'], 2)

        self.assertEqual(ag.results['b']['avg'], 5)
        self.assertEqual(ag.results['b']['max'], 5)
        self.assertEqual(ag.results['b']['min'], 5)
        self.assertEqual(ag.results['b']['count'], 1)



class Result:
    def __init__(self, database=None, params=None, elapsed=None, test=None):
        self.database = database
        self.params = params
        self.elapsed = elapsed
        self.test = test



from __future__ import absolute_import
from loveseat.couch_test import CouchTest
from loveseat.couch_specs import CouchReadSpec, CouchAllDocsSpec, CouchViewSpec


class Suite:
    def __init__(self, name, slug, tests):
        self.name = name
        self.slug = slug

        self.tests = []

        for test in tests:
            if test['test'] == 'read':
                self.tests.append(CouchTest(test['test'], spec=CouchReadSpec(test)))
            elif test['test'] == 'all_docs':
                self.tests.append(CouchTest(test['test'], spec=CouchAllDocsSpec(test)))
            elif test['test'] == 'view':
                self.tests.append(CouchTest(test['test'], spec=CouchViewSpec(test)))

    def run(self):
        results = []

        for test in self.tests:
            results.append(test.run())

        return results



from __future__ import absolute_import
import jsonobject as jo
from loveseat.couch_runners import read, all_docs, view


class CouchDatabaseSpec(jo.JsonObject):
    url = jo.StringProperty(required=True)
    slug = jo.StringProperty()
    params = jo.base.DefaultProperty()
    headers = jo.base.DefaultProperty()


class CouchSpec(jo.JsonObject):
    name = jo.StringProperty(required=True)
    test = jo.StringProperty(required=True)
    databases = jo.ListProperty(CouchDatabaseSpec)
    repeat = jo.IntegerProperty(default=10)
    params = jo.base.DefaultProperty()
    headers = jo.base.DefaultProperty()

    def __init__(self, obj=None, **kwargs):
        database_specs = []
        databases = (obj or kwargs).get('databases')

        for database in databases:
            spec = {
                'params': (obj or kwargs).get('params', {}),
                'headers': (obj or kwargs).get('headers', {})
            }
            if isinstance(database, unicode) or isinstance(database, str):
                spec.update({'url': database, 'slug': database})
                database_specs.append(spec)
            else:
                spec.update(database)
                if 'slug' not in spec:
                    spec['slug'] = spec['url']
                database_specs.append(spec)

        (obj or kwargs)['databases'] = database_specs

        super(CouchSpec, self).__init__(obj, **kwargs)


class CouchReadSpec(CouchSpec):

    ids = jo.ListProperty(required=True)

    def __call__(self):
        for db in self.databases:
            yield read(db.url, self.ids, params=self.params, headers=self.headers, slug=db.slug)


class CouchAllDocsSpec(CouchSpec):

    def __call__(self):
        for db in self.databases:
            yield all_docs(db.url, params=self.params, slug=db.slug)


class CouchViewSpec(CouchSpec):
    view = jo.StringProperty(required=True)

    def __call__(self):
        for db in self.databases:
            yield view(db.url, self.view, params=self.params, headers=self.headers, slug=db.slug)



from __future__ import absolute_import

from loveseat.test_runner import TestRunner


def main():
    from loveseat.cli import parser
    args = parser.parse_args()

    runner = TestRunner(args.suitefile)
    runner.run()



from __future__ import absolute_import
from __future__ import division
import uuid
import sys
import random
import string
import math


# Builds a doc of a certain size.
def build_doc(size):
    doc = {}
    n_attrs = 5

    for i in xrange(5):
        doc[uuid.uuid4()] = ''.join(
            random.choice(string.ascii_uppercase + string.digits) for _ in xrange(math.floor(size / n_attrs))
        )

    return doc



from __future__ import absolute_import

try:
    from setuptools import setup, find_packages
except ImportError:
    from distutils.core import setup, find_packages

import loveseat

setup(
    name='loveseat',
    version=loveseat.__version__,
    description='Benchmark library for testing CouchDB',
    author=loveseat.__author__,
    author_email='dev@dimagi.com',
    url='http://github.com/dimagi/loveseat',
    license=loveseat.__licence__,
    packages=find_packages(),
    entry_points={
        'console_scripts': [
            'loveseat = loveseat.__main__:main'
        ]
    },
    install_requires=[
        'requests',
        'ndg-httpsclient',
        'pyopenssl',
        'pyasn1',
        'clint',
        'jsonobject',
        'simplejson',
    ],
)



from __future__ import absolute_import
from loveseat.aggregated_result import AggregatedResult


class CouchTest(object):
    def __init__(self, test, spec=None):
        self.test = test
        self.spec = spec

    def agg_result(self):
        return AggregatedResult(self.spec.name)

    def run(self):
        iterations = self.spec.repeat
        agg_result = self.agg_result()

        while iterations:
            agg_result.add_results(list(self.spec()))
            iterations -= 1

        return agg_result



from collections import defaultdict


class AggregatedResult(object):
    """
    Keeps track of various statistics for Result objects. It separates statistics
    based on database name.
    """
    def __init__(self, name):
        self.name = name
        self.results = defaultdict(lambda: {
            'count': 0,
            'avg': None,
            'min': None,
            'max': None,
        })

    def add_result(self, result):
        entry = self.results[result.database]

        microseconds = result.elapsed.microseconds
        entry['avg'] = ((entry['avg'] or 0) * entry['count'] + microseconds) \
            / (entry['count'] + 1)

        entry['count'] += 1
        if microseconds > entry['max']:
            entry['max'] = microseconds
        if microseconds < entry['min'] or not entry['min']:
            entry['min'] = microseconds

    def add_results(self, results):
        for result in results:
            self.add_result(result)

    def __unicode__(self):
        return self.name

    def __str__(self):
        return self.__unicode__()
